// CFccSampleDlg.cpp : implementation file
//
#include "stdafx.h"
using namespace std;

#include "FccSampleVersionInfo.h"
#include "FccSample.h"
#include "FccSampleDlg.h"

#include "Profiler.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// CFccSampleDlg dialog

CFccSampleDlg::CFccSampleDlg(CWnd* pParent /*=NULL*/)
: CDialog(CFccSampleDlg::IDD, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CFccSampleDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_EDIT1, m_log);
	DDX_Control(pDX, IDC_BUTTON4, m_button);
}

BEGIN_MESSAGE_MAP(CFccSampleDlg, CDialog)
#if defined(_DEVICE_RESOLUTION_AWARE) && !defined(WIN32_PLATFORM_WFSP)
	ON_WM_SIZE()
#endif
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_BUTTON1, &CFccSampleDlg::OnBnClickedButtonExtract)
	ON_BN_CLICKED(IDC_BUTTON2, &CFccSampleDlg::OnBnClickedButtonIdentify)
	ON_BN_CLICKED(IDC_BUTTON3, &CFccSampleDlg::OnBnClickedButtonOptions)
	ON_BN_CLICKED(IDC_BUTTON4, &CFccSampleDlg::OnBnClickedButtonAbout)
	ON_BN_CLICKED(IDC_BUTTON5, &CFccSampleDlg::OnBnClickedButtonClearDB)
	ON_WM_CLOSE()
	ON_WM_PAINT()
END_MESSAGE_MAP()


static void ErrorLogInt(CEdit &log, const wchar_t *message, int number, bool addEndLine = true)
{
	wchar_t buffer1[1024], buffer2[1024];

	if (addEndLine)
	{
		swprintf(buffer1, _T("\r\n%s"), message);
		swprintf(buffer2, buffer1, number);
	}
	else
	{
		swprintf(buffer2, message, number);
	}

	CString text;
	log.GetWindowTextW(text);

	int length = text.GetLength();
	log.SetSel(length, length);
	log.ReplaceSel(buffer2);
}

static void ErrorLogDouble(CEdit &log, const wchar_t *message, double number, bool addEndLine = true)
{
	wchar_t buffer1[1024], buffer2[1024];

	if (addEndLine)
	{
		swprintf(buffer1, _T("\r\n%s"), message);
		swprintf(buffer2, buffer1, number);
	}
	else
	{
		swprintf(buffer2, message, number);
	}

	CString text;
	log.GetWindowTextW(text);

	int length = text.GetLength();
	log.SetSel(length, length);
	log.ReplaceSel(buffer2);
}

// CFccSampleDlg message handlers

BOOL CFccSampleDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

#ifdef WIN32_PLATFORM_WFSP
	if (!m_dlgCommandBar.Create(this) ||
		!m_dlgCommandBar.InsertMenuBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create CommandBar\n");
		return FALSE;      // fail to create
	}
#endif // WIN32_PLATFORM_WFSP

#if _WIN32_WCE >= 0x501
	m_cameraSupported = true;
#else
	m_cameraSupported = false;
#endif

	m_image = NULL;
	m_grayscale = NULL;
	m_recordCount = 0;
	m_records = NULL;

	CoInitializeEx(NULL, COINIT_MULTITHREADED);

	NResult result;

	// initialized extractor
	result = FcceCreate(&m_extractor);
	if (NFailed(result))
	{
		return FALSE;
	}
	result = FcceGetMaxTemplateSize(m_extractor, &m_bufferSize);
	if (NFailed(result))
	{
		return FALSE;
	}

	// initialize matcher
	result = FccmCreate(&m_matcher);
	if (NFailed(result))
	{
		return FALSE;
	}

	EnumerateComponents(true);

	// load templates from current folder
	LoadRecords();
	ErrorLogInt(m_log, _T("loaded %d face records"), m_recordCount);

	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CFccSampleDlg::EnumerateComponents(bool log)
{
	m_aboutDialog.ClearComponents();

	// add components to about box
	NLibraryInfo li;
	m_aboutDialog.AddComponent(FCCSAMPLE_FILE_NAME);
	NCoreGetInfo(&li);
	m_aboutDialog.AddComponent(&li, _T(""));
	NImagesGetInfo(&li);
	m_aboutDialog.AddComponent(&li, _T(""));
	NTemplateGetInfo(&li);
	m_aboutDialog.AddComponent(&li, _T(""));
	FcceGetInfo(&li);
	m_aboutDialog.AddComponent(&li, FcceIsRegistered() ? _T("Yes") : _T("No"));
	if (log)
	{
		ErrorLogInt(m_log, li.Title, 0, false);
		ErrorLogInt(m_log, _T(", Version "), 0, false);
		ErrorLogInt(m_log, _T("%d"), li.VersionMajor, false);
		ErrorLogInt(m_log, _T(".%d"), li.VersionMinor, false);
		ErrorLogInt(m_log, _T(".%d"), li.VersionBuild, false);
		ErrorLogInt(m_log, _T(".%d"), li.VersionRevision, false);
	}
	FccmGetInfo(&li);
	m_aboutDialog.AddComponent(&li, FccmIsRegistered() ? _T("Yes") : _T("No"));
	if (log)
	{
		ErrorLogInt(m_log, li.Title, 0);
		ErrorLogInt(m_log, _T(", Version "), 0, false);
		ErrorLogInt(m_log, _T("%d"), li.VersionMajor, false);
		ErrorLogInt(m_log, _T(".%d"), li.VersionMinor, false);
		ErrorLogInt(m_log, _T(".%d"), li.VersionBuild, false);
		ErrorLogInt(m_log, _T(".%d"), li.VersionRevision, false);
	}
}

#if defined(_DEVICE_RESOLUTION_AWARE) && !defined(WIN32_PLATFORM_WFSP)
void CFccSampleDlg::OnSize(UINT /*nType*/, int cx, int cy)
{
	// relayout dialog
	DRA::RelayoutDialog(
		AfxGetInstanceHandle(), this->m_hWnd, MAKEINTRESOURCE(IDD_FCCSAMPLE_DIALOG));

	// resize log to take all available space
	RECT rectDialog = {0, 0, cx, cy};
	ClientToScreen(&rectDialog);

	RECT rectLog, rectButton;
	m_log.GetWindowRect(&rectLog);
	m_button.GetWindowRect(&rectButton);
	rectLog.left = 3;
	rectLog.top = rectButton.bottom + 3;
	rectLog.right = rectDialog.right - 3;
	rectLog.bottom = rectDialog.bottom - 3;
	ScreenToClient(&rectLog);
	m_log.MoveWindow(&rectLog);
}
#endif

void CFccSampleDlg::OnClose()
{
	ClearRecords();
	ClearImages();
	FcceFree(m_extractor);
	FccmFree(m_matcher);
	CoUninitialize();

	CDialog::OnClose();
}

void CFccSampleDlg::ClearImages()
{
	if (m_image != NULL)
	{
		NImageFree(m_image);
		m_image = NULL;
	}
	if (m_grayscale != NULL)
	{
		NImageFree(m_grayscale);
		m_grayscale = NULL;
	}
}

void CFccSampleDlg::SetImage(HNImage image, HNImage grayscale)
{
	ClearImages();

	m_image = image;
	m_grayscale = grayscale;
	m_details.FaceAvailable = NFalse;
	m_details.EyesAvailable = NFalse;
}

void CFccSampleDlg::ClearRecords()
{
	for (int i = 0; i < m_recordCount; ++i)
	{
		free(m_records[i].Template);
		m_records[i].Template = NULL;
	}

	free(m_records);
	m_recordCount = 0;
}

void CFccSampleDlg::LoadRecords()
{
	ClearRecords();

	ifstream fi(fccDatabase, ios::binary);
	if (!fi.is_open())
	{
		return;
	}

	fi.read((char *) &m_recordCount, sizeof(int));
	m_records = (FaceRecord *) malloc(sizeof(FaceRecord) * m_recordCount);
	for (int i = 0; i < m_recordCount; ++i)
	{
		size_t sizeId;

		fi.read((char *) &sizeId, sizeof(size_t));
		fi.read((char *) m_records[i].Id, sizeof(wchar_t) * sizeId);
		m_records[i].Id[sizeId] = '\0';
		fi.read((char *) &m_records[i].Size, sizeof(NSizeType));
		m_records[i].Template = malloc(m_records[i].Size);
		fi.read((char *) m_records[i].Template, m_records[i].Size);
	}

	fi.close();
}

void CFccSampleDlg::SaveRecords()
{
	ofstream fo(fccDatabase, ios::binary);
	
	fo.write((char *) &m_recordCount, sizeof(int));
	for (int i = 0; i < m_recordCount; ++i)
	{
		size_t sizeId = wcslen(m_records[i].Id);

		fo.write((char *) &sizeId, sizeof(size_t));
		fo.write((char *) m_records[i].Id, sizeof(wchar_t) * sizeId);
		fo.write((char *) &m_records[i].Size, sizeof(NSizeType));
		fo.write((char *) m_records[i].Template, m_records[i].Size);
	}

	fo.close();
}

static bool NImageLoad(CString filename, HNImage *pHImage, HNImage *pHGrayscale)
{
	NResult result;
	//NUInt width, height;

	// load image from file
	result = NImageCreateFromFile(filename, 0, pHImage);
	if (NFailed(result))
	{
		return false;
	}

	// resize when image is too big
	/*
	result = NImageGetWidth(*pHImage, &width);
	result = NImageGetHeight(*pHImage, &height);
	if (width > height)
	{
		if (width > 640)
		{
			height = (int) ((double) 640 / (double) width * (double) height + 0.5);
			width = 640;
		}
	}
	else
	{
		if (height > 640)
		{
			width = (int) ((double) 640 / (double) height * (double) width + 0.5);
			height = 640;
		}
	}
	*/

	// convert image to grayscale
	result = NImageCreateFromImage(npfGrayscale, 0, *pHImage, pHGrayscale);
	if (NFailed(result))
	{
		return false;
	}

	return true;
}

void CFccSampleDlg::OnPaint()
{
	CPaintDC dc(this); // device context for painting

	if (m_image != NULL)
	{
		PAINTSTRUCT ps;
		RECT dstRect = {0, 0, 90, 120};
		CDC *dc = BeginPaint(&ps);
		NResult result;
		unsigned int width, height;

		result = NImageGetWidth(m_grayscale, &width);
		result = NImageGetHeight(m_grayscale, &height);

		if (width > height)
		{
			int temp = dstRect.right + 30;
			dstRect.right = dstRect.bottom + 40;
			dstRect.bottom = temp;
		}
		RECT buttonRect;
		m_button.GetClientRect(&buttonRect);
		int borderX = buttonRect.right + 6;
		int borderY = buttonRect.top + 3;
		dstRect.left += borderX;
		dstRect.top += borderY;
		dstRect.right += borderX;
		dstRect.bottom += borderY;

		NHandle handle;
		BmpSaveImageToHBitmap(m_image, &handle);

		HDC hdcMem;
		hdcMem = CreateCompatibleDC(dc->m_hDC);
		SelectObject(hdcMem, handle);
		StretchBlt(
			dc->m_hDC, dstRect.left, dstRect.top, dstRect.right - dstRect.left, dstRect.bottom - dstRect.top,
			hdcMem, 0, 0, width, height, SRCCOPY);
		DeleteObject(hdcMem);
		DeleteObject(handle);

		if (m_details.FaceAvailable)
		{
			double widthRatio = (double) (dstRect.right - dstRect.left) / width;
			double heightRatio = (double) (dstRect.bottom - dstRect.top) / height;
			RECT faceRect;

			faceRect.left = borderX + (int) (m_details.Face.Rectangle.X * widthRatio + 0.5);
			faceRect.top = borderY + (int) (m_details.Face.Rectangle.Y * heightRatio + 0.5);
			faceRect.right = (int) (faceRect.left + m_details.Face.Rectangle.Width * widthRatio + 0.5);
			faceRect.bottom = (int) (faceRect.top + m_details.Face.Rectangle.Height * heightRatio + 0.5);

			CBrush brushG(RGB(0, 255, 0));
			dc->FrameRect(&faceRect, &brushG);
			brushG.DeleteObject();

			if (m_details.EyesAvailable)
			{
				CPen penG(PS_SOLID, 1, RGB(0, 255, 0));
				CBrush brushK(RGB(128, 128, 128));
				CPen *oldPen = dc->SelectObject(&penG);
				CBrush *oldBrush = dc->SelectObject(&brushK);
				int radius = (int) (0.1 * (m_details.Eyes.Second.X - m_details.Eyes.First.X) * widthRatio + 0.5);
				int cx = borderX + (int) (m_details.Eyes.First.X * widthRatio + 0.5);
				int cy = borderY + (int) (m_details.Eyes.First.Y * heightRatio + 0.5);

				dc->Ellipse(cx - radius, cy - radius, cx + radius, cy + radius);
				cx = borderX + (int) (m_details.Eyes.Second.X * widthRatio + 0.5);
				cy = borderY + (int) (m_details.Eyes.Second.Y * heightRatio + 0.5);
				dc->Ellipse(cx - radius, cy - radius, cx + radius, cy + radius);

				dc->SelectObject(oldPen);
				dc->SelectObject(oldBrush);
				penG.DeleteObject();
				brushK.DeleteObject();
			}
		}

		EndPaint(&ps);
	}
}

HNImage CFccSampleDlg::GetImage(int from)
{
	CString filename;

	// get image from camera or from file
	SetImage(NULL, NULL);
	if (from == FCCSAMPLE_FROM_FILE)
	{
#ifndef WIN32_PLATFORM_WFSP
		// from file
		CFileDialog dialog(TRUE, 0, 0, 0, _T("All Images (*.jpg;*.bmp;*.png;*.tif)|*.jpg;*.bmp;*.png;*.tif||"));
		if (dialog.DoModal() != IDOK)
		{
			return NULL;
		}

		filename = dialog.GetPathName();
#endif
	}
#if _WIN32_WCE >= 0x501
	else if (from == FCCSAMPLE_FROM_CAMERA)
	{
		// from camera
		SHCAMERACAPTURE shcc;
		HRESULT         hReturn;

		// Specify the arguments of SHCAMERACAPTURE
		ZeroMemory(&shcc, sizeof(shcc));
		shcc.cbSize             = sizeof(shcc);
		shcc.hwndOwner          = this->m_hWnd;
		shcc.pszInitialDir      = NULL;
		shcc.pszDefaultFileName = _T("FccSample.jpg");
		shcc.pszTitle           = NULL;
		shcc.StillQuality       = CAMERACAPTURE_STILLQUALITY_HIGH;
		shcc.VideoTypes         = CAMERACAPTURE_VIDEOTYPE_ALL;
		shcc.nResolutionWidth   = 0;
		shcc.nResolutionHeight  = 0;
		shcc.nVideoTimeLimit    = 0;
		shcc.Mode               = CAMERACAPTURE_MODE_STILL;

		// Call SHCameraCapture() function
		hReturn = SHCameraCapture(&shcc);

		// Check the return codes of the SHCameraCapture() function
		bool doExtraction = false;
		switch (hReturn)
		{
		case S_OK:
			// The method completed successfully.
			ErrorLogInt(m_log, _T("image captured"), 0);
			doExtraction = true;
			break;
		case S_FALSE:
			// The user canceled the Camera Capture dialog box.
			ErrorLogInt(m_log, _T("canceled capturing"), 0);
			break;
		case E_INVALIDARG:
			ErrorLogInt(m_log, _T("invalid arguments!"), 0);
			break;
		case E_OUTOFMEMORY:
			ErrorLogInt(m_log, _T("out of memory!"), 0);
			break;
		default:
			// An unknown error occurred.
			ErrorLogInt(m_log, _T("unknown return value!"), 0);
			break;
		}

		if (!doExtraction)
		{
			return NULL;
		}

		filename = shcc.szFile;
	}
#endif
	else
	{
		ErrorLogInt(m_log, _T("the device is not supported!"), 0);
		return NULL;
	}

	// load image
	HNImage image = NULL;
	HNImage grayscale = NULL;
	if (!NImageLoad(filename, &image, &grayscale))
	{
		ErrorLogInt(m_log, _T("failed to load image from file!"), 0);
		return NULL;
	}
	SetImage(image, grayscale);

	return grayscale;
}

void *CFccSampleDlg::GetTemplate(int from, const wchar_t *id, NSizeType *size)
{
	// extraction
	HNImage grayscale = GetImage(from);
	TimeIntervalCounter tc;
	NResult result;
	void *buffer = malloc(m_bufferSize);

	StartTICounter(&tc);
	result = FcceExtract(m_extractor, grayscale, &m_details, buffer, m_bufferSize, size);
	StopTICounter(&tc);
	if (NFailed(result) || (*size == 0))
	{
		free(buffer);

		if (id != NULL)
		{
			ErrorLogInt(m_log, id, 0);
			ErrorLogInt(m_log, _T(": extraction failed (result = %d)!"), result, false);
		}
		else
		{
			ErrorLogInt(m_log, _T("extraction failed (result = %d)!"), result);
		}
		ErrorLogInt(m_log, _T("template size = %d, "), *size);
		ErrorLogInt(m_log, _T("done in "), 0, false);
		ErrorLogInt(m_log, CString(tc.str), 0, false);
		return NULL;
	}
	if (id != NULL)
	{
		ErrorLogInt(m_log, id, 0);
		ErrorLogInt(m_log, _T(": extracted in "), 0, false);
		ErrorLogInt(m_log, CString(tc.str), 0, false);
	}
	else
	{
		ErrorLogInt(m_log, _T("extracted in "), 0);
		ErrorLogInt(m_log, CString(tc.str), 0, false);
	}

	/**
	// show face detection details
	if (m_details.FaceAvailable)
	{
		ErrorLogInt(m_log, _T("found face:"), 0);
		wchar_t pbuffer[1024];

		swprintf(pbuffer, _T("    location = (%d, %d)"),
		m_details.Face.Rectangle.X, m_details.Face.Rectangle.Y);
		ErrorLogInt(m_log, pbuffer, 0);

		swprintf(pbuffer, _T("    size = (%d, %d)"),
		m_details.Face.Rectangle.Width, m_details.Face.Rectangle.Height);
		ErrorLogInt(m_log, pbuffer, 0);

		swprintf(pbuffer, _T("    confidence = %.2f"),
		m_details.Face.Confidence);
		ErrorLogInt(m_log, pbuffer, 0);
	}
	if (m_details.EyesAvailable)
	{
		ErrorLogInt(m_log, _T("found eyes:"), 0);
		wchar_t pbuffer[1024];

		swprintf(pbuffer, _T("    first = (%d, %d, %.2f)"),
		m_details.Eyes.First.X, m_details.Eyes.First.Y, m_details.Eyes.FirstConfidence);
		ErrorLogInt(m_log, pbuffer, 0);

		swprintf(pbuffer, _T("    second = (%d, %d, %.2f)"),
		m_details.Eyes.Second.X, m_details.Eyes.Second.Y, m_details.Eyes.SecondConfidence);
		ErrorLogInt(m_log, pbuffer, 0);
	}
	/**/

	return buffer;
}

void CFccSampleDlg::OnBnClickedButtonExtract()
{
	CString id;
	int from, mode;

	// read id
	if (m_enrollDialog.DoModal() != IDOK)
	{
		return;
	}
	from = m_enrollDialog.m_from;
	mode = m_enrollDialog.m_mode;
	id = m_enrollDialog.m_id;

	int count = 1;
	void **buffers = NULL;
	NSizeType *sizes = NULL;
	void *buffer = NULL;
	NSizeType size = 0;
	int gotCount = 0;
	wchar_t strBuffer[1024];
	NResult result;
	NBool stopped = NFalse;

	switch (mode)
	{
	case FCCSAMPLE_SINGLE_IMAGE:
		buffer = GetTemplate(from, id.GetString(), &size);
		if (buffer != NULL)
		{
			NByte faceQuality;
			NLRecordGetQualityMem(buffer, m_bufferSize, &faceQuality);
			ErrorLogInt(m_log, _T("got template of quality %d"), faceQuality);
		}
		break;
	case FCCSAMPLE_IMAGE_SEQ:
		buffer = malloc(m_bufferSize);

		// use the same number of images as for generalization
		count = m_optionsDialog.m_genCount;
		result = FcceExtractStart(m_extractor, count);
		if (NFailed(result))
		{
			ErrorLogInt(m_log, _T("function ExtractStart() failed (result = %d)!"), result);
		}
		else
		{
			ErrorLogInt(m_log, _T("started extraction from %d images"), count);
		}
		for (int i = 0; i < count; ++i)
		{
			HNImage grayscale = GetImage(from);
			TimeIntervalCounter tc;

			wsprintf(strBuffer, _T("%s[%d]"), id.GetString(), i);
			StartTICounter(&tc);
			result = FcceExtractNext(m_extractor, grayscale,
				&m_details, buffer, m_bufferSize, &size, &stopped);
			StopTICounter(&tc);
			if (NSucceeded(result))
			{
				ErrorLogInt(m_log, strBuffer, 0);
				ErrorLogInt(m_log, _T(": extracted in "), 0, false);
				ErrorLogInt(m_log, CString(tc.str), 0, false);
			}
			if (NFailed(result) || stopped)
			{
				break;
			}
		}
		ErrorLogInt(m_log, _T("finished extraction from %d images"), count);

		if (NFailed(result) || (size == 0))
		{
			ErrorLogInt(m_log, _T("extraction failed (result = %d)!"), result);
			ErrorLogInt(m_log, _T("template size = %d"), size);

			free(buffer);
			free(buffers);
			free(sizes);
			return;
		}
		else
		{
			NByte faceQuality;
			NLRecordGetQualityMem(buffer, m_bufferSize, &faceQuality);
			ErrorLogInt(m_log, _T("got template of quality %d"), faceQuality);
		}
		break;
	case FCCSAMPLE_GENERALIZATION:
		count = m_optionsDialog.m_genCount;
		buffers = (void **) malloc(sizeof(void *) * count);
		sizes = (NSizeType *) malloc(sizeof(NSizeType) * count);

		ErrorLogInt(m_log, _T("started generalization from %d images"), count);
		for (int i = 0; i < count; ++i)
		{
			wsprintf(strBuffer, _T("%s[%d]"), id.GetString(), i);
			buffers[gotCount] = GetTemplate(from, strBuffer, &size);
			sizes[gotCount] = size;
			if (buffers[gotCount] == NULL)
			{
				if (mode == FCCSAMPLE_GENERALIZATION)
				{
					ErrorLogInt(m_log, _T("failed to acquire %d face for generalization"), i);
				}
				else
				{
					free(buffers);
					free(sizes);
					return;
				}
			}
			else
			{
				++gotCount;
			}
		}

		size = 0;
		buffer = malloc(m_bufferSize);
		result = FcceGeneralize(m_extractor,
			gotCount, (const void **) buffers, sizes,
			buffer, m_bufferSize, &size);
		ErrorLogInt(m_log, _T("finished generalization from %d "), gotCount);
		ErrorLogInt(m_log, _T("of %d images"), count, false);

		if (NFailed(result) || (size == 0))
		{
			ErrorLogInt(m_log, _T("generalization failed (result = %d)!"), result);
			ErrorLogInt(m_log, _T("template size = %d"), size);

			free(buffer);
			free(buffers);
			free(sizes);
			return;
		}
		else
		{
			NByte faceQuality;
			NLRecordGetQualityMem(buffer, m_bufferSize, &faceQuality);
			ErrorLogInt(m_log, _T("got template of quality %d"), faceQuality);
		}
		free(buffers);
		free(sizes);

		break;
	}

	// save template to db
	m_records = (FaceRecord *) realloc(m_records, sizeof(FaceRecord) * (m_recordCount + 1));
	wcscpy_s(m_records[m_recordCount].Id, maxIdLength, id.GetString());
	m_records[m_recordCount].Size = size;
	m_records[m_recordCount].Template = buffer;
	++m_recordCount;
	SaveRecords();
}

void CFccSampleDlg::OnBnClickedButtonIdentify()
{
	int from;

	// read id
	if (m_cameraSupported)
	{
		if (m_identifyDialog.DoModal() != IDOK)
		{
			return;
		}
		from = m_identifyDialog.m_from;
	}
	else
	{
		from = FCCSAMPLE_FROM_FILE;
	}

	NSizeType size;
	void *buffer = GetTemplate(from, NULL, &size);
	if (buffer == NULL)
	{
		return;
	}
	else
	{
		NByte faceQuality;
		NLRecordGetQualityMem(buffer, m_bufferSize, &faceQuality);
		ErrorLogInt(m_log, _T("got template of quality %d"), faceQuality);
	}

	// perform identification
	TimeIntervalCounter tc;
	bool identified = false;
	int count = 0;

	ErrorLogInt(m_log, _T("started identification"), 0);
	StartTICounter(&tc);
	NResult result = FccmIdentifyStart(m_matcher, buffer, size);
	PauseTICounter(&tc);
	for (int i = 0; i < m_recordCount; ++i)
	{
		NDouble score = 0.0;
		ResumeTICounter(&tc);
		result = FccmIdentifyNext(m_matcher, m_records[i].Template, m_records[i].Size, &score);
		PauseTICounter(&tc);

		++count;

		if (score > 0)
		{
			identified = true;
			ErrorLogInt(m_log, _T("    identified: "), 0);
			ErrorLogInt(m_log, m_records[i].Id, 0, false);
			ErrorLogDouble(m_log, _T(" (score = %.4f)"), score, false);
			break;
		}
	}
	ResumeTICounter(&tc);
	result = FccmIdentifyEnd(m_matcher);
	StopTICounter(&tc);
	ErrorLogInt(m_log, _T("finished identification"), 0);
	if (tc.mks > 0)
	{
		ErrorLogDouble(m_log, _T("matching speed: %.2f faces/second"),
			(double) count * 1000000.0 / (double) tc.mks);
	}
	else
	{
		ErrorLogInt(m_log, _T("identification took less than 1 mcs"), 0);
	}

	//if (identified)
	//{
	//	PlaySound(MAKEINTRESOURCE(IDR_WAVE1), AfxGetInstanceHandle(), SND_RESOURCE | SND_ASYNC);
	//}

	// free memory taken by template
	free(buffer);
}

void CFccSampleDlg::OnBnClickedButtonOptions()
{
	if (m_optionsDialog.DoModal() != IDOK)
	{
		return;
	}

	NResult result;

	result = FcceSetParameter(m_extractor, FCCEP_MIN_IOD, &m_optionsDialog.m_minIOD);
	if (NFailed(result))
	{
		ErrorLogInt(m_log, _T("failed to set FCCEP_MIN_IOD parameter!"), 0);
		result = FcceGetParameter(m_extractor, FCCEP_MIN_IOD, &m_optionsDialog.m_minIOD);
	}

	result = FcceSetParameter(m_extractor, FCCEP_MAX_IOD, &m_optionsDialog.m_maxIOD);
	if (NFailed(result))
	{
		ErrorLogInt(m_log, _T("failed to set FCCEP_MAX_IOD parameter!"), 0);
		result = FcceGetParameter(m_extractor, FCCEP_MAX_IOD, &m_optionsDialog.m_maxIOD);
	}

	NByte faceQualityThreshold = (NByte)m_optionsDialog.m_faceQualityThreshold;
	result = FcceSetParameter(m_extractor, FCCEP_FACE_QUALITY_THRESHOLD, &faceQualityThreshold);
	if (NFailed(result))
	{
		ErrorLogInt(m_log, _T("failed to set FCCEP_FACE_QUALITY_THRESHOLD parameter!"), 0);
		result = FcceGetParameter(m_extractor, FCCEP_FACE_QUALITY_THRESHOLD, &m_optionsDialog.m_maxIOD);
		m_optionsDialog.m_faceQualityThreshold = faceQualityThreshold;
	}

	result = FcceSetParameter(m_extractor, FCCEP_GENERALIZATION_THRESHOLD, &faceQualityThreshold);
	if (NFailed(result))
	{
		ErrorLogInt(m_log, _T("failed to set FCCEP_GENERALIZATION_THRESHOLD parameter!"), 0);
		result = FcceGetParameter(m_extractor, FCCEP_GENERALIZATION_THRESHOLD, &m_optionsDialog.m_genThreshold);
	}

	result = FccmSetParameter(m_matcher, FCCMP_MATCHING_THRESHOLD, &m_optionsDialog.m_matchThreshold);
	if (NFailed(result))
	{
		ErrorLogInt(m_log, _T("failed to set FCCMP_MATCHING_THRESHOLD parameter!"), 0);
		result = FcceGetParameter(m_extractor, FCCMP_MATCHING_THRESHOLD, &m_optionsDialog.m_matchThreshold);
	}
}

void CFccSampleDlg::OnBnClickedButtonAbout()
{
	EnumerateComponents(false);
	m_aboutDialog.DoModal();
}

void CFccSampleDlg::OnBnClickedButtonClearDB()
{
	ClearRecords();
	SaveRecords();
	ErrorLogInt(m_log, _T("cleared face database"), 0);
}
